@@ -5,6 +5,7 @@ trainControl <- function(
                          returnData = TRUE,
                          returnResamp = "final",
                          p = .75,
+                         classProbs = FALSE,
                          summaryFunction = defaultSummary,
                          selectionFunction = "best",
                          index = NULL,
@@ -21,6 +22,7 @@ trainControl <- function(
        returnData = returnData,
        returnResamp = returnResamp,
        p = p,
+       classProbs = classProbs,
        summaryFunction = summaryFunction,
        selectionFunction = selectionFunction,
        index = index,
